<?php

/* DISCLAIMER *****************************************************************
*                                                                             *
* The Purple Yin Guestbook code is distributed without any warranty. All      *
* warranties, expressed or implied are hereby disclaimed. This includes but   *
* is not limited to warranties of merchantability or fitness for a particular *
* purpose. The reselling of this code is strictly prohibited.                 *
*                                                                             *
*******************************************************************************

/* PURPLE YIN GUESTBOOK INFORMATION *******************************************
*                                                                             *
* URL:           www.purpleyin.com                                            *
* AUTHOR:        Thomas Phan                                                  *
*                                                                             *
******************************************************************************/

define('MAGIC_QUOTES_SYBASE',false);

define('GUESTBOOK_NAME','Purple Yin Guestbook Version 1.1.1');

define('DATA_FILE_HEADER',"<?php header('location: sign.php'); exit();?>\r\n");

/*
Check for undefined functions.
Ensure compatibility with PHP versions >= 4.1.0
file_get_contents(string $filename): Available in PHP4 >= 4.3.0 and PHP5.
*/
if(!function_exists('file_get_contents'))
{
	function file_get_contents($filename)
	{
		return implode('',file($filename));
	}
}
/* End: Check for undefined functions. */

/*
Define additional functions.
handleLineEndings(string $string):
cleanFormData(string $string):
cleanFileInput(string $string):
prepFileOutput(string $string):
string2JavaScript(string $string):
removeFromArray(string $needle,array $haystack):
pyg_htmlspecialchars(string $string):
pyg_unhtmlspecialchars(string $string):
*/
function handleLineEndings($string)
{
	/*
	Windows, Unix and Macintosh all use different line endings ("\r\n", "\n", "\r").
	Convert all line endings to "\r\n".
	*/
	$string=str_replace("\r\n","\n",$string);
	$string=str_replace("\r","\n",$string);
	$string=str_replace("\n","\r\n",$string);
	
	return$string;
}

function cleanFormData($string)
{
	if(get_magic_quotes_gpc()==1)
	{
		if(MAGIC_QUOTES_SYBASE)
		{
			$string=str_replace("''","'",$string);
		}
		else
		{
			$string=stripslashes($string);
		}
	}
	
	return handleLineEndings($string);
}

function cleanFileInput($string)
{
	if(get_magic_quotes_runtime()==1)
	{
		if(MAGIC_QUOTES_SYBASE)
		{
			$string=str_replace("''","'",$string);
		}
		else
		{
			$string=stripslashes($string);
		}
	}
	
	return handleLineEndings($string);
}

function prepFileOutput($string)
{
	if(get_magic_quotes_runtime()==1)
	{
		if(MAGIC_QUOTES_SYBASE)
		{
			$string=str_replace("'","''",$string);
		}
		else
		{
			$string=str_replace("\\","\\\\",$string);
		}
	}
	
	return handleLineEndings($string);
}

function string2JavaScript($string)
{
	$temp=array();
	
	for($i=0;$i<strlen($string);$i++)
	{
		$temp[]="'".str_replace("'","\\'",substr($string,$i,1))."'";
	}
	
	return'<script language="JavaScript" type="text/JavaScript">document.write('.implode('+',$temp).');</script>';
}

function removeFromArray($needle,$haystack)
{
	$temp=array();

	foreach($haystack as $item)
	{
		if($item!=$needle)
		{
			$temp[]=$item;
		}
	}

	return$temp;
}

function pyg_htmlspecialchars($string)
{
	$pattern=array(
		'#&(?!(\#[0-9]+;))#',
		'#<#',
		'#>#',
		"#'#",
		'#"#'
	);
	
	$replacement=array(
		'&amp;',
		'&lt;',
		'&gt;',
		'&#039;',
		'&quot;'
	);

	return preg_replace($pattern,$replacement,$string);
}

function pyg_unhtmlspecialchars($string)
{
	$search=array(
		'&amp;',
		'&lt;',
		'&gt;',
		'&#039;',
		'&quot;'
	);
	
	$replace=array(
		'&',
		'<',
		'>',
		'\'',
		'"'
	);

	return str_replace($search,$replace,$string);
}
/* End: Define additional functions. */

if(isset($_POST))
{
	$_POST=array_map('cleanFormData',$_POST);
}
else
{
	$_POST=array();
}

if(isset($_GET))
{
	$_GET=array_map('cleanFormData',$_GET);
}
else
{
	$_GET=array();
}

$options=new PYGOPTIONS();

/******************************************************************************
* CLASS: PYGTEMPLATES
******************************************************************************/
class PYGTEMPLATES
{
	var $templates;

	function PYGTEMPLATES()
	{
		$templatesFile=new PYGDATAFILE('temp.default.php');

		$templates=explode('<separator>',$templatesFile->getString());

		$this->templates=array(
			'errorHeader'=>	$templates[0],
			'perError'=>	$templates[1],
			'errorFooter'=>	$templates[2],
			'sign'=>		$templates[3],
			'thanks'=>		$templates[4],
			'viewHeader'=>	$templates[5],
			'viewEntry'=>	$templates[6],
			'viewFooter'=>	$templates[7],
			'reply'=>		$templates[8],
			'login'=>		$templates[9]
		);
	}

	function get($templateName)
	{
		return$this->templates[$templateName];
	}

	function set($templateName,$template)
	{
		$templatesFile=new PYGDATAFILE('temp.default.php');

		$this->templates[$templateName]=$template;

		$templatesFile->setString(implode('<separator>',$this->templates));

		$templatesFile->save();
	}
}

/******************************************************************************
* CLASS: PYGLIST
******************************************************************************/
class PYGLIST
{
	var $filename;
	var $theList;
	
	function PYGLIST($filename)
	{
		$this->filename=$filename;

		$this->populateList();
	}

	function populateList()
	{

		$dataFile=new PYGDATAFILE($this->filename);

		$this->theList=$dataFile->getList();
	}

	function get()
	{
		return$this->theList;
	}

	function set($newList)
	{
		$this->theList=$newList;
		
		$this->save();
	}

	function add($newItem)
	{
		/*
		Remove whitespace from the beginning and end of item.
		Add the item if it is not already in the list and 
		is neither empty nor whitespace.
		*/
		$newItem=trim($newItem);
		
		if(!in_array($newItem,$this->theList)&&strlen($newItem)>0)
		{
			$this->theList[]=$newItem;
			
			$this->save();
		}
	}

	function remove($removeMe)
	{
		$this->theList=removeFromArray($removeMe,$this->theList);
		
		$this->save();
	}

	function inList($item)
	{
		return in_array($item,$this->theList);
	}

	function save()
	{
		$dataFile=new PYGDATAFILE($this->filename);
		
		$dataFile->setList($this->theList);
		
		$dataFile->save();
	}
}

/******************************************************************************
* CLASS: PYGBANNEDIPS
******************************************************************************/
class PYGBANNEDIPS extends PYGLIST
{
	function PYGBANNEDIPS()
	{
		$this->filename='banned.php';
		
		$this->populateList();
	}
	
	function isBanned($IP)
	{
		return $this->inList($IP);
	}
}

/******************************************************************************
* CLASS: BADWORDS
******************************************************************************/
class PYGBADWORDS extends PYGLIST
{
	function PYGBADWORDS()
	{
		$this->filename='badWords.php';
	
		$this->populateList();
	}

	function findBadWords($testString)
	{
		$whitespace=array(
			"\t",
			"\n",
			"\r",
			"\0",
			"\x0B"
		);
		
		/* Convert to lowercase. Bad words are stored in lowercase. */
		$testString=strtolower($testString);
		
		/* Replace whitespace characters with a space. */
		foreach($whitespace as $replace)
		{
			$testString=str_replace($replace,' ',$testString);
		}
		
		if(count(array_intersect(explode(' ',$testString),$this->theList))==0)
		{
			return false;
		}
		else
		{
			return true;
		}
	}

	function add($badWord)
	{
		/*
		Remove whitespace from the beginning and end of the word.
		Add the word if it is not already in the list and 
		is neither empty nor whitespace. Store as lowercase.
		*/
		$badWord=strtolower(trim($badWord));
		
		if(!in_array($badWord,$this->theList)&&strlen($badWord)>0)
		{
			$this->theList[]=$badWord;
			
			$this->save();
		}
	}
}

/******************************************************************************
* CLASS: BADSTRINGS
******************************************************************************/
class PYGBADSTRINGS extends PYGLIST
{
	function PYGBADSTRINGS()
	{
		$this->filename='badStrings.php';
		
		$this->populateList();
	}

	function findBadStrings($testString)
	{
		/* Convert to lowercase. Bad strings are stored in lowercase. */
		$testString=strtolower($testString);
		
		foreach($this->theList as $badString)
		{
			if(is_int(strpos($testString,$badString)))
			{
				return true;
			}
		}
		
		return false;
	}
	
	function add($badString)
	{
		/*
		Remove whitespace from the beginning and end of the word.
		Add the word if it is not already in the list and 
		is neither empty nor whitespace. Store as lowercase.
		*/
		$badString=strtolower(trim($badString));
		
		if(!in_array($badString,$this->theList)&&strlen($badString)>0)
		{
			$this->theList[]=$badString;
			
			$this->save();
		}
	}
}

/******************************************************************************
* CLASS: PYGFIELDS
******************************************************************************/
class PYGFIELDS
{
	var $fields;

	function PYGFIELDS()
	{
		$this->fields=array();

		$fieldsFile=new PYGDATAFILE('fields.php');

		foreach($fieldsFile->getList() as $fieldData)
		{
			$data=explode('<separator>',$fieldData);

			$this->fields[$data[0]]=$data[1];
		}
	}

	function getNames()
	{
		return array_keys($this->fields);
	}

	function toggleRequired($fieldName)
	{
		$this->fields[$fieldName]=($this->fields[$fieldName]+1)%2;

		$this->save();
	}

	function required($fieldName)
	{
		if(isset($this->fields[$fieldName])&&$this->fields[$fieldName]==1)
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	function save()
	{
		$fieldsFile=new PYGDATAFILE('fields.php');

		$temp=array();

		foreach(array_keys($this->fields) as $fieldName)
		{
			$temp[]=$fieldName.'<separator>'.$this->fields[$fieldName];
		}

		$fieldsFile->setList($temp);

		$fieldsFile->save();
	}
}

/******************************************************************************
* CLASS: PYGPASSWORD
******************************************************************************/
class PYGPASSWORD
{
	var $password;

	/*
	Set class variable, $password, to encrypted password stored in file
	named "password.php". The password is stored encrypted in
	password.php.
	*/
	function PYGPASSWORD()
	{
		$passwordFile=new PYGDATAFILE('password.php');

		$passwordList=$passwordFile->getList();

		if(count($passwordList)==1)
		{
			$this->password=$passwordList[0];
		}
		else
		{
			$this->password='';
		}
	}

	function set($password)
	{
		$this->password=crypt($password);

		$passwordFile=new PYGDATAFILE('password.php');

		$passwordList=$passwordFile->getList();

		$passwordList[0]=$this->password;

		$passwordFile->setList($passwordList);

		$passwordFile->save();
	}

	function reset()
	{
		$this->password='';
		
		$passwordFile=new PYGDATAFILE('password.php');
		
		$passwordList=array();
		
		$passwordFile->setList($passwordList);
		
		$passwordFile->save();
	}

	function get()
	{
		return $this->password;
	}

	/*
	Return true if an encrypted password is stored in password.php and
	false if otherwise.
	*/
	function present()
	{
		if(strlen($this->password)==0)
		{
			return false;
		}
		else
		{
			return true;
		}
	}

	/*
	Encrypt the password passed to this function with the stored
	password as the salt and return it.
	*/
	function crypt($password)
	{
		return crypt($password,$this->password);
	}
}

/******************************************************************************
* CLASS: PYGOPTIONS
******************************************************************************/
class PYGOPTIONS
{
	var $options;

	function PYGOPTIONS()
	{
		$optionsFile=new PYGDATAFILE('options.php');

		$optionsList=$optionsFile->getList();

		$this->options=array();

		$this->options['emptyGoTo']=			$optionsList[0];
		$this->options['entriesPerPage']=		$optionsList[1];
		$this->options['spanPages']=			$optionsList[2];
		$this->options['entryOrder']=			$optionsList[3];
		$this->options['maxWordLength']=		$optionsList[4];
		$this->options['smiliesEnabled']=		$optionsList[5];
		$this->options['emailNotification']=	$optionsList[6];
		$this->options['adminEmail']=			$optionsList[7];
		$this->options['timeOffset']=			$optionsList[8];
		$this->options['maxEntryLength']=		$optionsList[9];
		$this->options['passwordProtect']=		$optionsList[10];
		$this->options['allowHTML']=			$optionsList[11];
		$this->options['day1']=					$optionsList[12];
		$this->options['day2']=					$optionsList[13];
		$this->options['day3']=					$optionsList[14];
		$this->options['day4']=					$optionsList[15];
		$this->options['day5']=					$optionsList[16];
		$this->options['day6']=					$optionsList[17];
		$this->options['day7']=					$optionsList[18];
		$this->options['month1']=				$optionsList[19];
		$this->options['month2']=				$optionsList[20];
		$this->options['month3']=				$optionsList[21];
		$this->options['month4']=				$optionsList[22];
		$this->options['month5']=				$optionsList[23];
		$this->options['month6']=				$optionsList[24];
		$this->options['month7']=				$optionsList[25];
		$this->options['month8']=				$optionsList[26];
		$this->options['month9']=				$optionsList[27];
		$this->options['month10']=				$optionsList[28];
		$this->options['month11']=				$optionsList[29];
		$this->options['month12']=				$optionsList[30];
		$this->options['privatePlaceholders']=	$optionsList[31];
		$this->options['requiredFieldError']=	$optionsList[32];
		$this->options['longEntryError']=		$optionsList[33];
		$this->options['IPError']=				$optionsList[34];
		$this->options['badWordError']=			$optionsList[35];
		$this->options['badStringError']=		$optionsList[36];
		$this->options['entryNumbersOrder']=	$optionsList[37];
		$this->options['validationRequired']=	$optionsList[38];
		$this->options['signGoTo']=				$optionsList[39];
		$this->options['maxLines']=				$optionsList[40];
		$this->options['maxLinesError']=		$optionsList[41];
		$this->options['hostNameByIP']=			$optionsList[42];
	}

	function set($optionName,$optionValue)
	{
		$this->options[$optionName]=$optionValue;

		$this->save();
	}

	function get($optionName)
	{
		return$this->options[$optionName];
	}

	function getMonth($monthNumber)
	{
		return$this->options['month'.$monthNumber];
	}

	function getDay($dayNumber)
	{
		return$this->options['day'.$dayNumber];
	}

	function toggle($optionName)
	{
		$this->set($optionName,($this->options[$optionName]+1)%2);

		$this->save();
	}

	function save()
	{
		$optionsFile=new PYGDATAFILE('options.php');

		$optionsFile->setList($this->options);

		$optionsFile->save();
	}
}

/******************************************************************************
* CLASS: PYGSMILIES
******************************************************************************/
class PYGSMILIES
{
	var $smilies;

	function PYGSMILIES()
	{
		$smiliesFile=new PYGDATAFILE('smilies.php');

		$smilies=array();

		foreach($smiliesFile->getList() as $smileyData)
		{
			$chunks=explode('<separator>',$smileyData);

			$smiley=array();

			$smiley['id']=			$chunks[0];
			$smiley['emoticon']=	$chunks[1];
			$smiley['filename']=	$chunks[2];
			$smiley['advanced']=	$chunks[3];

			$this->smilies[]=$smiley;
		}
	}

	function save()
	{
		$smiliesFile=new PYGDATAFILE('smilies.php');

		$temp=array();

		foreach($this->smilies as $smiley)
		{
			$temp[]=implode('<separator>',$smiley);
		}

		$smiliesFile->setList($temp);

		$smiliesFile->save();
	}

	function get()
	{
		return $this->smilies;
	}

	function set($smilies)
	{
		$this->smilies=$smilies;
		
		$this->save();
	}

	function add($emoticon,$filename,$advanced)
	{
		$maxID=0;
		
		foreach($this->get() as $smiley)
		{
			if($smiley['id']>$maxID)
			{
				$maxID=$smiley['id'];
			}
		}
	
		$smiley=array();

		$smiley['id']=++$maxID;
		$smiley['emoticon']=$emoticon;
		$smiley['filename']=$filename;
		$smiley['advanced']=$advanced;

		$this->smilies[]=$smiley;

		$this->save();
	}

	function remove($ID)
	{
		$tempArray=array();

		foreach($this->smilies as $smiley)
		{
			if($smiley['id']!=$ID)
			{
				$tempArray[]=$smiley;
			}
		}

		$this->smilies=$tempArray;

		$this->save();
	}
}

/******************************************************************************
* CLASS: PYGDB
******************************************************************************/
class PYGDB
{
	var $rows;
	var $changes;
	
	function PYGDB()
	{
		$this->changes=array();

		$this->rows=array();

		$DBFile=new PYGDATAFILE('db.php');

		foreach($DBFile->getList() as $item)
		{
			$columns=explode('<>',$item);

			$row=array();

			$columnNames=$this->getColumnNames();

			for($i=0;$i<count($columnNames);$i++)
			{
				$row[$columnNames[$i]]=pyg_unhtmlspecialchars(str_replace('<br />',"\r\n",$columns[$i]));
			}

			$this->rows[$row['id']]=$row;
		}
	}

	function update($row)
	{
		if(func_num_args()==1)
		{
			$this->rows[$row['id']]=$row;
			
			$this->changes[$row['id']]=$row;
		}
		else
		{
			/* Order of arguments: id,column name, column value */
			$this->rows[func_get_arg(0)][func_get_arg(1)]=func_get_arg(2);
			
			$this->changes[func_get_arg(0)]=$this->rows[func_get_arg(0)];
		}
		
		$this->save();
	}

	function getRows($order)
	{
		$this->sort($order);
		
		$temp=array();
		
		/* Populate $temp with active rows. */
		foreach($this->rows as $row)
		{
			if($row['status']==1)
			{
				$temp[$row['id']]=$row;
			}
		}

		return$temp;
	}

	function getDeletedRows($order)
	{
		$this->sort($order);
		
		$temp=array();
		
		/* Populate $temp with deleted rows. */
		foreach($this->rows as $row)
		{
			if($row['status']==2)
			{
				$temp[$row['id']]=$row;
			}
		}

		return$temp;
	}

	function getRowsAwaitingValidation($order)
	{
		$this->sort($order);
		
		$temp=array();
		
		/* Populate $temp with rows awaiting validation. */
		foreach($this->rows as $row)
		{
			if($row['status']==0)
			{
				$temp[$row['id']]=$row;
			}
		}

		return$temp;
	}

	function getAllRows($order)
	{
		$this->sort($order);
		
		return$this->rows;
	}

	function insert($newRow)
	{
		$temp=explode(' ',microtime());

		$ID=strval($temp[0]+$temp[1]);

		$duplicateID=true;

		while($duplicateID)
		{
			$IDFound=false;

			foreach($this->rows as $row)
			{
				if($row['id']==$ID)
				{
					$IDFound=true;

					break;
				}
			}

			if($IDFound)
			{
				$temp=explode(' ',microtime());

				$ID=strval($temp[0]+$temp[1]);
			}
			else
			{
				$duplicateID=false;
			}
		}
		
		$newRow['id']=$ID;
		
		$this->rows[$ID]=$newRow;
		
		$this->changes[$ID]=$newRow;

		$this->save();
	}

	function select($ID)
	{
		return$this->rows[$ID];
	}

	function sort($order)
	{
		$rowsKeyedByTimestamps=array();

		foreach($this->rows as $row)
		{
			/*
			dateAndTime is needed to sort chronologically, but
			dateAndTime is not unique. 'id' is unique.
			The concatenation of 'dateAndTime' and 'id' is unique
			and can also be used to sort chronologically.
			*/
			$rowsKeyedByTimestamps[$row['dateAndTime'].$row['id']]=$row;
		}

		switch($order)
		{
			case'chronological':
				ksort($rowsKeyedByTimestamps);
			break;

			case'reversechronological':
				krsort($rowsKeyedByTimestamps);
			break;
		}

		$this->rows=array();

		foreach($rowsKeyedByTimestamps as $rowKeyedByTimestamps)
		{
			$this->rows[$rowKeyedByTimestamps['id']]=$rowKeyedByTimestamps;
		}
	}

	function delete($ID)
	{
		$this->rows[$ID]['status']=2;
		
		$this->changes[$ID]=$this->rows[$ID];
		
		$this->save();
	}

	function getColumnNames()
	{
		return array(
			'id',
			'dateAndTime',
			'name',
			'email',
			'homepage',
			'location',
			'comments',
			'ip',
			'private',
			'reply',
			'extraField1',
			'extraField2',
			'extraField3',
			'extraField4',
			'extraField5',
			'status' /* 0=Awaiting Validation; 1=Active; 2=Deleted */
		);
	}

	function save()
	{
		$appendString='';

		$columnNames=$this->getColumnNames();

		foreach($this->changes as $row)
		{
			for($i=0;$i<count($columnNames)-1;$i++)
			{
				$temp=pyg_htmlspecialchars($row[$columnNames[$i]]);
				
				$temp=prepFileOutput($temp);
				
				$temp=str_replace("\r\n",'<br />',$temp);
				
				$appendString.=$temp.'<>';
			}
			
			$temp=pyg_htmlspecialchars($row[$columnNames[count($columnNames)-1]]);
			
			$temp=prepFileOutput($temp);

			$temp=str_replace("\r\n",'<br />',$temp);
			
			$appendString.=$temp."\r\n";
		}
		
		$fp=fopen('lock.php','ab');
		
		while(!flock($fp,2+4)){}
		
		$fp2=fopen('db.php','ab');
		
		fwrite($fp2,$appendString);
		
		fclose($fp2);
		
		while(!flock($fp,3+4)){}

		fclose($fp);
		
		$this->changes=array();
	}
}

/******************************************************************************
* CLASS: PYGLANGUAGEFILE
******************************************************************************/
class PYGLANGUAGEFILE
{
	var $phrases;

	function PYGLANGUAGEFILE($filename)
	{
		$this->phrases=array();
		
		$languageFile=new PYGDATAFILE($filename);
		
		$phrasesList=$languageFile->getList();
		
		$this->phrases['day1']=					$phrasesList[0];
		$this->phrases['day2']=					$phrasesList[1];
		$this->phrases['day3']=					$phrasesList[2];
		$this->phrases['day4']=					$phrasesList[3];
		$this->phrases['day5']=					$phrasesList[4];
		$this->phrases['day6']=					$phrasesList[5];
		$this->phrases['day7']=					$phrasesList[6];
		$this->phrases['month1']=				$phrasesList[7];
		$this->phrases['month2']=				$phrasesList[8];
		$this->phrases['month3']=				$phrasesList[9];
		$this->phrases['month4']=				$phrasesList[10];
		$this->phrases['month5']=				$phrasesList[11];
		$this->phrases['month6']=				$phrasesList[12];
		$this->phrases['month7']=				$phrasesList[13];
		$this->phrases['month8']=				$phrasesList[14];
		$this->phrases['month9']=				$phrasesList[15];
		$this->phrases['month10']=				$phrasesList[16];
		$this->phrases['month11']=				$phrasesList[17];
		$this->phrases['month12']=				$phrasesList[18];
		$this->phrases['requiredFieldError']=	$phrasesList[19];
		$this->phrases['longEntryError']=		$phrasesList[20];
		$this->phrases['IPError']=				$phrasesList[21];
		$this->phrases['badWordError']=			$phrasesList[22];
		$this->phrases['badStringError']=		$phrasesList[23];
		$this->phrases['maxLinesError']=		$phrasesList[24];
	}
	
	function get($phraseName)
	{
		return$this->phrases[$phraseName];
	}
	
	function getPhraseNames()
	{
		return array_keys($this->phrases);
	}
}
/******************************************************************************
* CLASS: PYGDATAFILE
******************************************************************************/
class PYGDATAFILE
{
	var $contents;
	var $filename;

	function PYGDATAFILE($filename)
	{
		$this->filename=$filename;

		$fp=fopen('lock.php','rb');
		
		while(!flock($fp,1+4)){}

		$this->contents=file_get_contents($filename);

		$this->contents=cleanFileInput($this->contents);

		/* Remove data file header. */
		$this->contents=str_replace(DATA_FILE_HEADER,'',$this->contents);
		
		while(!flock($fp,3+4)){}

		fclose($fp);
	}

	function getString()
	{
		return$this->contents;
	}

	function setString($string)
	{
		$this->contents=$string;
	}

	function getList()
	{
		if($this->contents=='')
		{
			return array();
		}
		else
		{
			$lines=explode("\r\n",$this->contents);
			
			array_pop($lines);
		}
		
		/*
		List items should not have whitespace at the beginning or end.
		Is this a good thing?
		*/
		return array_map('trim',$lines);
	}

	function setList($newList)
	{
		if(count($newList)>0)
		{
			$this->contents=implode("\r\n",$newList)."\r\n";
		}
		else
		{
			$this->contents='';
		}
	}

	function save()
	{
		$fp=fopen('lock.php','ab');
		
		while(!flock($fp,2+4)){}

		$fp2=fopen($this->filename,'wb');

		fwrite($fp2,prepFileOutput(DATA_FILE_HEADER.$this->getString()));

		fclose($fp2);
		
		while(!flock($fp,3+4)){}

		fclose($fp);
	}
}

/******************************************************************************
* CLASS: PYGTURINGIMAGE
******************************************************************************/
class PYGTURINGIMAGE
{

	/*
	Turning String Image Generator
	Copyright (c) 2004 Colin Viebrock <colin@viebrock.ca>
	Usage of the works is permitted provided that this instrument is retained with the works, so that any entity that uses the works is notified of this instrument.
	DISCLAIMER: THE WORKS ARE WITHOUT WARRANTY.
	*/

	function printImage()
	{
		/* Seed with microseconds. */
		list($usec,$sec)=explode(' ',microtime());
		
		mt_srand((float)$sec+((float)$usec*100000));
	
		/* Define the characters that can make up the Turing string. No "l", "1", "o" or "0". */
		$src='abcdefghijkmnpqrstuvwxyz23456789';
		
		/* Use either all upper or lower-case. */
		/* Use lower-case only.
		if(mt_rand(0,1))
		{
			$src=strtoupper($src);
		}
		*/
		
		$srclen=strlen($src)-1;
		
		/* Length of Turing string. */
		$length=8;
		
		/* Path to font file. */
		$font=realpath('arial.ttf');
		
		/* Font size range. */
		$min_font_size=12;
		$max_font_size=20;
		
		/* Angle range. */
		$min_angle=-15;
		$max_angle=15;
		
		/* Character padding. */
		$char_padding=2;
		
		$turingString='';
		
		$data=array();
		
		$image_width=$image_height=0;
		
		/* Build the data array of the characters, size, placement, etc. */
		for($i=0;$i<$length;$i++)
		{
			$char=substr($src,mt_rand(0,$srclen),1);
			
			$turingString.=$char;
		
			$size=mt_rand($min_font_size,$max_font_size);
			
			$angle=mt_rand($min_angle,$max_angle);
		
			$bbox=imagettfbbox($size,$angle,$font,$char);
		
			$char_width=max($bbox[2],$bbox[4])-min($bbox[0],$bbox[6]);
			
			$char_height=max($bbox[1],$bbox[3])-min($bbox[7],$bbox[5]);
		
			$image_width+=$char_width+$char_padding;
			
			$image_height=max($image_height,$char_height);
			
			$data[]=array(
				'char'=>	$char,
				'size'=>	$size,
				'angle'=>	$angle,
				'height'=>	$char_height,
				'width'=>	$char_width
			);
		}
		
		/* Calculate the final image size, adding some padding. */
		$x_padding=12;
		
		$image_width+=($x_padding*2);
		
		$image_height=($image_height*1.5)+2;
		
		/* Build the image. */
		$im=imagecreate($image_width,$image_height);
		
		/* Allocate colors. */
		$r=51*mt_rand(4,5);
		$g=51*mt_rand(4,5);
		$b=51*mt_rand(4,5);

		$color_bg=imagecolorallocate($im,$r,$g,$b);
		
		$r=51*mt_rand(3,4);
		$g=51*mt_rand(3,4);
		$b=51*mt_rand(3,4);

		$color_line0=imagecolorallocate($im,$r,$g,$b);
		
		$r=51*mt_rand(3,4);
		$g=51*mt_rand(3,4);
		$b=51*mt_rand(3,4);
		
		$color_line1=imagecolorallocate($im,$r,$g,$b);
		
		$r=51*mt_rand(1,2);
		$g=51*mt_rand(1,2);
		$b=51*mt_rand(1,2);
		
		$color_text=imagecolorallocate($im,$r,$g,$b);
		
		$color_border=imagecolorallocate($im,0,0,0);
		
		/* Make random background lines. */
		for($i=0;$i<10;$i++)
		{
			$c='color_line'.($i%2);
			
			$lx=mt_rand(0,$image_width+$image_height);
			
			$lw=mt_rand(0,3);
			
			if($lx>$image_width)
			{
				$lx-=$image_width;
				
				imagefilledrectangle($im,0,$lx,$image_width-1,$lx+$lw,$$c);
			}
			else
			{
				imagefilledrectangle($im,$lx,0,$lx+$lw,$image_height-1,$$c);
			}
		}
		
		/* Output each character */
		$pos_x=$x_padding+($char_padding/2);
		
		foreach($data as $d)
		{
			$pos_y=(($image_height+$d['height'])/2);
			
			imagettftext($im,$d['size'],$d['angle'],$pos_x,$pos_y,$color_text,$font,$d['char']);
			
			$pos_x+=$d['width']+$char_padding;
		}
		
		/* Create border. */
		imagerectangle($im,0,0,$image_width-1,$image_height-1,$color_border);
		
		/* Display image. */
		header('Content-type: image/jpeg');
				
		imagejpeg($im,NULL,100);
		
		imagedestroy($im);
		
		return$turingString;
	}
}
?>